"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var DirectorySearcher_1 = require("../Searcher/DirectorySearcher");
var ExpressRouteBuilder_1 = require("../RouteBuilder/ExpressRouteBuilder");
var express_1 = __importDefault(require("express"));
var pkg_up_1 = __importDefault(require("pkg-up"));
var path_1 = require("path");
var regex_parser_1 = __importDefault(require("regex-parser"));
var escape_string_regexp_1 = __importDefault(require("escape-string-regexp"));
var url_join_1 = __importDefault(require("url-join"));
var route_join_1 = require("../../util/route-join");
var RouteSearcher_1 = require("../Searcher/RouteSearcher");
var MiddlewareSearcher_1 = require("../Searcher/MiddlewareSearcher");
var ExpressBootstrap = /** @class */ (function () {
    function ExpressBootstrap(app) {
        this.app = app;
    }
    ExpressBootstrap.prototype.boot = function (options) {
        var _this = this;
        var directorySearcher = new DirectorySearcher_1.DirectorySearcher(options);
        var routeSearcher = new RouteSearcher_1.RouterSearcher(directorySearcher, options);
        var middlewareSearcher = new MiddlewareSearcher_1.MiddlewareSearcher(directorySearcher, options);
        var directoryResults = routeSearcher.search();
        var middlewareResults = middlewareSearcher.search();
        var middlewareRoutes = new ExpressRouteBuilder_1.ExpressRouteBuilder().getRoutes(middlewareResults);
        var routes = new ExpressRouteBuilder_1.ExpressRouteBuilder().getRoutes(directoryResults);
        var middlewareConfig = new ExpressRouteBuilder_1.ExpressHandlerBuilder().build(middlewareRoutes);
        var routeConfig = new ExpressRouteBuilder_1.ExpressHandlerBuilder().build(routes);
        var routeList = {};
        //create all middlewares
        middlewareConfig.forEach(function (routeHandles) {
            var routePath = route_join_1.normalizeRoute(routeHandles.route);
            var router = routeList[routePath] || (routeList[routePath] = express_1.default.Router());
            routeHandles.config.forEach(function (middlewareHandle) {
                var middlewareHandlers = require(middlewareHandle.required);
                Object.entries(middlewareHandlers).forEach(function (_a) {
                    var name = _a[0], handler = _a[1];
                    if (handler) {
                        router.use(handler);
                    }
                });
            });
        });
        routeConfig.forEach(function (routeHandles) {
            var routePath = route_join_1.normalizeRoute(routeHandles.route);
            var router = routeList[routePath] || (routeList[routePath] = express_1.default.Router());
            routeHandles.config.forEach(function (handle) {
                if (options.verbose) {
                    logRouteBootInfo(options, routeHandles, handle);
                }
                router.all(route_join_1.normalizeRoute(handle.path), function (req, res, next) {
                    var handler = require(handle.required);
                    var handlerName = Object.keys(handler)[0];
                    if (handler[handlerName]) {
                        if (options.verbose) {
                            logRouteInfo(routeHandles, handle, handlerName);
                        }
                        handler[handlerName](req, res, next);
                    }
                    else {
                        next();
                    }
                    if (options.live) {
                        var dir_1 = path_1.dirname(pkg_up_1.default.sync());
                        Object.entries(require.cache).forEach(function (_a) {
                            var key = _a[0];
                            if (regex_parser_1.default(escape_string_regexp_1.default(dir_1)).test(key) && !/node_modules/.test(key)) {
                                delete require.cache[key];
                            }
                        });
                    }
                });
            });
        });
        Object.entries(routeList).forEach(function (_a) {
            var route = _a[0], router = _a[1];
            _this.app.use(route, router);
        });
    };
    return ExpressBootstrap;
}());
exports.ExpressBootstrap = ExpressBootstrap;
function logRouteBootInfo(options, routeHandles, handle) {
    console.log(("\n" + url_join_1.default(options.route || "/", routeHandles.route, handle.path) + "\n               ").trim());
}
function logRouteInfo(route, handle, name) {
    console.log("\n" + ("\n" + route.route + "\n    " + handle.path + "\n        (" + name + "): " + handle.filepath + "\n                        ").trim());
}
