"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = require("path");
var leven_1 = __importDefault(require("leven"));
var url_join_1 = __importDefault(require("url-join"));
var regex_parser_1 = __importDefault(require("regex-parser"));
var escape_string_regexp_1 = __importDefault(require("escape-string-regexp"));
var ExpressRouteBuilder = /** @class */ (function () {
    function ExpressRouteBuilder() {
    }
    ExpressRouteBuilder.prototype.getRoutes = function (searchResults, store) {
        if (store === void 0) { store = {}; }
        searchResults.forEach(function (result) {
            var currentPath = result.relativePath;
            do {
                var collections = store[currentPath] = store[currentPath] || [];
                collections.push(result);
                currentPath = path_1.dirname(currentPath);
            } while (currentPath !== "/");
        });
        var routeWithoutParams = Object.entries(store)
            .filter(function (_a) {
            var route = _a[0], config = _a[1];
            return !route.match(regex_parser_1.default("/" + escape_string_regexp_1.default(":") + "[A-z0-9]/i"));
        });
        var routeWithoutParamsSorted = routeWithoutParams.sort(function (_a, _b) {
            var k1 = _a[0], v1 = _a[1];
            var k2 = _b[0], v2 = _b[1];
            return leven_1.default(k1, k2);
        }) || routeWithoutParams;
        routeWithoutParamsSorted.forEach(function (_a, index1) {
            var key = _a[0], routeContainer = _a[1];
            routeWithoutParamsSorted.forEach(function (_a, index2) {
                var key2 = _a[0], routeContainer2 = _a[1];
                if (index1 === index2) {
                    return;
                }
                var copy1 = routeContainer.slice(0);
                var copy2 = routeContainer2.slice(0);
                copy1.forEach(function (routeConfig, index) {
                    if (routeContainer2.indexOf(routeConfig) == -1) {
                        return;
                    }
                    var furtherDown = index1 < index2;
                    var betterPlacement = copy2.length > copy1.length;
                    //ensure route config is only located in one optimal path
                    if (furtherDown && betterPlacement) {
                        routeContainer.splice(routeContainer.indexOf(routeConfig), 1);
                    }
                    else {
                        routeContainer2.splice(routeContainer2.indexOf(routeConfig), 1);
                    }
                });
            });
        });
        return routeWithoutParamsSorted.reduce(function (a, _a) {
            var _b;
            var key = _a[0], value = _a[1];
            if (value.length > 0) {
                return Object.assign(a, (_b = {}, _b[key] = value, _b));
            }
            else {
                return a;
            }
        }, {});
    };
    return ExpressRouteBuilder;
}());
exports.ExpressRouteBuilder = ExpressRouteBuilder;
var ExpressHandlerBuilder = /** @class */ (function () {
    function ExpressHandlerBuilder() {
    }
    ExpressHandlerBuilder.prototype.build = function (routes) {
        var appSettings = Object.entries(routes).map(function (_a) {
            var route = _a[0], routeOptions = _a[1];
            var config = routeOptions.map(function (option) {
                var path = url_join_1.default("/", option.relativePath.replace(regex_parser_1.default("/^" + escape_string_regexp_1.default(route) + "/"), "")) || "/";
                var filepath = option.filepath;
                return {
                    path: path,
                    filepath: filepath,
                    required: path_1.resolve(filepath).replace(option.ext, "")
                };
            });
            return {
                route: route,
                config: config
            };
        });
        return appSettings;
    };
    return ExpressHandlerBuilder;
}());
exports.ExpressHandlerBuilder = ExpressHandlerBuilder;
